/*Basic software to run the E39 Prius inverter / Siemens motor conversion with ISA can shunt
 * Uses a repurposed GS450H vcu v2 as hardware.

 * WiFi connection on USART2 at 19200 baud
 * 
 * ISA CAN shunt connectivity. Note V2 hardware only. 

 * Copyright 2020 D.Maguire
 * openinverter.org
 * evbmw.com
 * 
 */


#include <Metro.h>
#include <due_can.h>  //https://github.com/collin80/due_can
#include <due_wire.h> //https://github.com/collin80/due_wire
#include <DueTimer.h>  //https://github.com/collin80/DueTimer
#include <Wire_EEPROM.h> //https://github.com/collin80/Wire_EEPROM
#include <ISA.h>  //isa can shunt library




#define COOLANT  33 //oil pump relay
#define Tacho_Sig  2  //oil pump pwm
#define InvPower    34  //inverter relay
#define PRECONT  50   //Out1
#define NEGCONT  47   //TransSL1
#define POSCONT  44   //TransSL2  
#define DCDC   45     //TransSP

#define IGNON   6     //IN1
#define CHGREQ   7    //IN2
#define Brake_In   62


#define HVPreset    310 //voltage at which to enable main contactor



////////////////Global variables ////////////////////////////////////////////////////////////
word RPM;

////////////////////////////////////////////////////////////////////////////////////////////////////
CAN_FRAME outframe;  //A structured variable according to due_can library for transmitting CAN data.







 
Metro timer_diag = Metro(1100);
Metro timer_200Tasks = Metro(200);
Metro timer_Frames100 = Metro(100);
Metro timer_RPM = Metro(5);
Metro timer_Frames10 = Metro(10);


bool CARON;       //flag for car on or off status
bool CHGON;       //flag for charging on or off status
bool dash_status; //flag for dash on can command.
bool can_status;  //flag for turning off and on can sending.
     

ISA Sensor;  //Instantiate ISA Module Sensor object to measure current and voltage 

#define SerialDEBUG SerialUSB

void setup() {

   Can0.begin(CAN_BPS_500K);  //CAN bus for isa shunt
   Can1.begin(CAN_BPS_500K);  //
   Can1.attachCANInterrupt(Incoming); //

    Sensor.begin(0,500);  //Start ISA object on CAN 0 at 500 kbps
   

 //set initial conditions/////////////////
    CARON=false;
    dash_status=false;
    can_status=false;
    RPM=750;

//////////////////////////////////////////////   
  pinMode(13, OUTPUT);  //led
  pinMode(COOLANT, OUTPUT);  //Water pump control relay
  digitalWrite(COOLANT,LOW);  //turn off water pump
  

  pinMode(InvPower, OUTPUT);  //Inverter Relay 
  pinMode(PRECONT, OUTPUT);  //GP output one
  pinMode(NEGCONT,OUTPUT); //Trans solenoids
  pinMode(POSCONT,OUTPUT); //Trans solenoids
  pinMode(DCDC,OUTPUT); //Trans solenoids
  pinMode(Tacho_Sig,OUTPUT); //output to drive tacho

  digitalWrite(InvPower,LOW);  //turn off at startup
  digitalWrite(PRECONT,LOW);  //turn off at startup
  digitalWrite(NEGCONT,LOW);  //turn off at startup
  digitalWrite(POSCONT,LOW);  //turn off at startup
  digitalWrite(DCDC,LOW);  //turn off at startup
  digitalWrite(Tacho_Sig,LOW);  //tacho drive low at startup

  pinMode(IGNON,INPUT); //Input 1 ignition is on when high
  pinMode(CHGREQ,INPUT); //Input 2 charge request when high
  pinMode(Brake_In,INPUT); //Brake pedal input

 
  SerialDEBUG.begin(115200);
  SerialDEBUG.print("hello world!");

  Serial2.begin(19200); //setup serial 2 for wifi access
}



void HV_On()
{

  
}

void HV_Off()
{

 
}


void handle_wifi(){
/*
 * 
 * Routine to send data to wifi on serial 2
The information will be provided over serial to the esp8266 at 19200 baud 8n1 in the form :
vxxx,ixxx,pxxx,mxxxx,nxxxx,oxxx,rxxx,qxxx* where :

v=pack voltage (0-700Volts)
i=current (0-1000Amps)
p=power (0-300kw)
m=mg1 rpm (0-10000rpm)
n=mg2 rpm (0-10000rpm)
o=mg1 temp (-20 to 120C)
r=mg2 temp (-20 to 120C)
q=oil pressure (0-100%)
*=end of string
xxx=three digit integer for each parameter eg p100 = 100kw.
updates will be every 100ms approx.

v100,i200,p35,m3000,n4000,o20,r100,q50*
*/
  
//Serial2.print("v100,i200,p35,m3000,n4000,o20,r30,q50*"); //test string

digitalWrite(13,!digitalRead(13));//blink led every time we fire this interrrupt.

Serial2.print("v");//dc bus voltage
Serial2.print(Sensor.Voltage);//voltage derived from ISA shunt
Serial2.print(",i");//dc current
Serial2.print(Sensor.Amperes);//current derived from ISA shunt
Serial2.print(",p");//total motor power
Serial2.print(Sensor.KW);//Power value derived from ISA Shunt
Serial2.print(",m");//Front motor rpm
Serial2.print(RPM);
Serial2.print(",n");//Rear motor rpm
Serial2.print(RPM);
Serial2.print(",o");//Will be sent via CAN from inverter. Use ISA temp for now
Serial2.print(Sensor.Temperature);
Serial2.print(",r");//Will be sent via CAN from inverter. Use ISA temp for no
Serial2.print(Sensor.Temperature);
Serial2.print(",q");// pwm percent on oil pump
Serial2.print(Sensor.KWH);// change to kwh on wifi.
Serial2.print("*");// end of data indicator

}





/////////////////////////////////////////////////////////////////////////////////////////////////////
///////Handle incomming pt can messages from the car here
////////////////////////////////////////////////////////////////////////////////////////////////////
void Incoming (CAN_FRAME *frame){

    }



/////////////Send frames every 10ms and send/rexeive inverter control serial data ///////////////////////////////////////

void Frames10MS()
{
    if(timer_Frames10.check())
  {
    
  }
 
    
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////Send these frames every 100ms /////////////////////////////////////////
void Frames100MS()
{
      if(timer_Frames100.check())
  {
    
  }

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////



////////////Do these tasks every 200ms /////////////////////////////////////////
void Tasks200()
{
      if(timer_200Tasks.check())
  {

  if(digitalRead(IGNON)&& !CARON)  //if ignition switch is on but car is not turned on
  {

 // digitalWrite(InvPower,HIGH);  //turn on inverter and pas pump power
  digitalWrite(COOLANT,HIGH);  //turn on coolant pump
  digitalWrite(NEGCONT,HIGH);  //turn on hv precharge and negative side contactor
  digitalWrite(PRECONT,HIGH);  //turn on hv precharge and negative side contactor
  //CARON=true; //set car on status flag once precharge is commenced. Experiment....
  if (Sensor.Voltage>HVPreset)
    {
        digitalWrite(POSCONT,HIGH);  //turn on hv main once precharge complete
        digitalWrite(PRECONT,LOW);  //turn off precharge contactor
        digitalWrite(DCDC,HIGH);  //turn on dcdc converter
        digitalWrite(InvPower,HIGH);  //turn on inverter and pas pump power
        CARON=true; //set car on status flag once precharge is complete and main contactor is on
    }
  

  }


  if(digitalRead(IGNON)==0 && CARON)  //if ignition switch goes off and car is on
  {
  digitalWrite(DCDC,LOW);  //turn off dcdc converter
  delay(10);
  digitalWrite(PRECONT,LOW);  //turn off precharge contactor
  digitalWrite(POSCONT,LOW);  //turn off hv
  digitalWrite(NEGCONT,LOW);  //
  digitalWrite(InvPower,LOW);  //turn off inverter and pas pump power
  digitalWrite(COOLANT,LOW);  //turn off coolant pump
    CARON=false; // set car on flag to false once all off
  }

    
    
  }


  if(digitalRead(CHGREQ)&& !CHGON)  //if charge request line is high but we are not in charge mode
  {

  digitalWrite(COOLANT,HIGH);  //turn on coolant pump
  digitalWrite(NEGCONT,HIGH);  //turn on hv precharge and negative side contactor
  digitalWrite(PRECONT,HIGH);  //turn on hv precharge and negative side contactor
  
  if (Sensor.Voltage>HVPreset)
    {
        digitalWrite(POSCONT,HIGH);  //turn on hv main once precharge complete
        digitalWrite(PRECONT,LOW);  //turn off precharge contactor
        CHGON=true; //set charge on status flag once precharge is complete and main contactor is on
    }
  

  }



    if(digitalRead(CHGON)==0 && CHGON)  //if charge request line goes low and we are in charge mode...
  {
  delay(10);
  digitalWrite(PRECONT,LOW);  //turn off precharge contactor
  digitalWrite(POSCONT,LOW);  //turn off hv
  digitalWrite(NEGCONT,LOW);  //
  digitalWrite(COOLANT,LOW);  //turn off coolant pump
    CHGON=false; // set charge on flag to false once all off
  }
  

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////





void loop() {

  if(timer_diag.check())
  {
  handle_wifi();
  }

Tasks200(); //Run the 200ms tasks loop





  
}
